#%% Example Use of the emon api functions to load, process and write back data
##============================================================================
#Configuration information 
function_path='/home/<PATH TO EMON PYTHON FUNCCTIONS>'
api_url='http:/<IP FOR EMON>/feed/' 

api_key='<API KEY>' #this is read/write api key
max_url_len=6000 #maximum length of a URL - this is defined by emon, not clear where but 6000 seems safe
                 #if exceeded emon reponse is "414 Request-URI Too Long"
                 
save_file='cached_feed.pickle' #where the downloaded dataframe is saved.

from datetime import datetime, timezone, timedelta

import pandas as pd
import sys
import requests
sys.path.append(function_path)
import emon_api_functions_utc as emon

#Switches below determine allow data to be downloaded (and cached locally)

get_live_data=False                    #True to download data else load cached pickle file
write_dataframe2emon_bulk=True        # True to write back to emon via API 
simulate_write=False                 # True to not actually make the API call, used for run through of data

#Note the new feed has to be manually created with correct frequency BEFORE this script is run
old_feed_name='old_lounge_TV_kwh'
new_feed_name='lounge_TV_kwh'

#the start and end times for the data we want to extract and write back
#all times are done in UTC, doing in local /DST gets too messy
start_time=datetime(2023,1,3,00,00,tzinfo=timezone.utc)
end_time = datetime(2023,2,8,00,00,tzinfo=timezone.utc) 


#%% Read old feed from emon 
if get_live_data:

    
    #at upto 10 seconds data (max emon does) 1 day is a safe limit for readingc
    api_call_step=timedelta(days=1)
       
    #now loop through the data and build a list of all samples including na/Null ones
    api_query_start_time = start_time
    days_api_request_sw_lists=[]
   
    while (api_query_start_time < end_time):
        api_query_end_time = api_query_start_time+api_call_step
        print('working on : ',api_query_start_time.strftime('%Y-%m-%d %H:%M:%S'))
    
        d1_df= emon.dataframe_from_feed_single(api_query_start_time, api_query_end_time,old_feed_name, api_url, api_key)
        print('Total Samples :',len(d1_df),'  ',end='')
        
        
        days_api_request_sw_lists.append(d1_df) 
        api_query_start_time=api_query_end_time
        #time.sleep(1)
       
    print('Old Feed Name : ',old_feed_name)
    print('all downloaded - generating consolidated dataframe')
    main_df=pd.concat(days_api_request_sw_lists)
    
    print('Total Sample Count : ',len(main_df))
    main_df.dropna(inplace=True)
    print('ASamples Count after Null data drop ',len(main_df))
    
    print('Saving as a Pickle to : ',save_file)
    
    main_df.to_pickle(save_file)
else:
    print('Loading Save Pickle File : ',save_file)
    main_df=pd.read_pickle(save_file)

print('The data is loaded and in dataframe : main_df')
    
#data can now be manupulated with all the standard PANDAS tecniques 
#including resampling 
#e.g. for main_df=main_df.resample('1D').mean()

#Docs
#https://pandas.pydata.org/docs/reference/api/pandas.DataFrame.resample.html

#Loads more examples here:
#https://towardsdatascience.com/pandas-resample-tricks-you-should-know-for-manipulating-time-series-data-7e9643a7e7f3

#--------------------------------------------------------------------
#%%
#wNow Write processed data back to emon


if write_dataframe2emon_bulk:

    #IMPORTANT - the new feed has to be created and existing in Emon BEFORE running this script
   
    #TODO - the API can create feeds, add a faclity to create them.
    feed_meta_data=emon.get_all_feed_data_dataframe(api_url, api_key)
    new_feed_id_str=emon.feed_id_from_name(new_feed_name, feed_meta_data)

    if (new_feed_id_str=='-999'):
        print('Sorry New Feed Name ',new_feed_name,' is not found - Exiting')
        exit(-1)

    #the size of the chunks we can bulk upload is determined by the length of the string
    # (url) we call.  set to 2 hours and run simulated, if erros reduce time
    #not you can use minutes= and days= in the timdelta command if needed.  
    #TODO work out the size of a write and the number of rows we can do in one go.
    write_api_call_step= timedelta(hours=2) 
    
    #set loop variables for first run through
    extract_start_time=start_time
    response_text = '{"success":true}'
    
    while ((response_text == '{"success":true}') and (extract_start_time < end_time-write_api_call_step)):
      
        
        #generate chunk of data
        
        extract_end_time=extract_start_time+write_api_call_step
        print('Processing : '+extract_start_time.strftime('%Y-%m-%d %H:%M'),end='')
        d1w_df=main_df[extract_start_time:extract_end_time]
        if (len(d1w_df) > 0):
            #generate the bulk load GET string by starting the URL
            write_str=api_url+'insert.json?id='+new_feed_id_str+'&data=['
                
            #now loop through the pairs adding 
            for sample_time,data_row in d1w_df.iterrows():
                time_ms=emon.epoch_ms_from_datetime(sample_time)
                sample_val=data_row[old_feed_name]
                
                sample_val_str='{:.2f}'.format(sample_val)
                time_str=str(int(time_ms/1e3))
                sample_pair='['+time_str+','+sample_val_str+'],'
                
                write_str+=sample_pair
                
            
            #drop final_comma and add closing bracket and API
            write_str=write_str[:-1]+']&apikey='+api_key
            
            #data is in the url held in write_string.  Now write the data
            if (len(write_str) > max_url_len):
                print('Sorry Length of URL is too long')
                exit(-2)
            if simulate_write:
                print('Simulated Write!',end='')
                print(write_str,end='')
                response_text='{"success":true}'
            else:
                response=requests.get(write_str)
                response_text=response.text
            
            print(response_text)
            extract_start_time=extract_end_time
        else:
            print('No Samples!')
   
            extract_start_time=extract_end_time
    
    print('Finished! - Final Response Text : ',response_text)
